@@ -18,10 +18,7 @@ import copy
 import functools
 
 from oslo_log import log
-from oslo_log import versionutils
 
-from keystone.assignment.backends import base
-from keystone.assignment.role_backends import base as role_base
 from keystone.common import cache
 from keystone.common import dependency
 from keystone.common import driver_hints
@@ -68,37 +65,8 @@ class Manager(manager.Manager):
 
     def __init__(self):
         assignment_driver = CONF.assignment.driver
-        # If there is no explicit assignment driver specified, we let the
-        # identity driver tell us what to use. This is for backward
-        # compatibility reasons from the time when identity, resource and
-        # assignment were all part of identity.
-        if assignment_driver is None:
-            msg = _('Use of the identity driver config to automatically '
-                    'configure the same assignment driver has been '
-                    'deprecated, in the "O" release, the assignment driver '
-                    'will need to be expicitly configured if different '
-                    'than the default (SQL).')
-            versionutils.report_deprecated_feature(LOG, msg)
-            try:
-                identity_driver = dependency.get_provider(
-                    'identity_api').driver
-                assignment_driver = identity_driver.default_assignment_driver()
-            except ValueError:
-                msg = _('Attempted automatic driver selection for assignment '
-                        'based upon [identity]\driver option failed since '
-                        'driver %s is not found. Set [assignment]/driver to '
-                        'a valid driver in keystone config.')
-                LOG.critical(msg)
-                raise exception.KeystoneConfigurationError(msg)
         super(Manager, self).__init__(assignment_driver)
 
-        # Make sure it is a driver version we support, and if it is a legacy
-        # driver, then wrap it.
-        if isinstance(self.driver, base.AssignmentDriverV8):
-            self.driver = base.V9AssignmentWrapperForV8Driver(self.driver)
-        elif not isinstance(self.driver, base.AssignmentDriverV9):
-            raise exception.UnsupportedDriverVersion(driver=assignment_driver)
-
         self.event_callbacks = {
             notifications.ACTIONS.deleted: {
                 'domain': [self._delete_domain_assignments],
@@ -123,13 +91,6 @@ class Manager(manager.Manager):
         # Use set() to process the list to remove any duplicates
         return list(set([x['user_id'] for x in assignment_list]))
 
-    def _list_parent_ids_of_project(self, project_id):
-        if CONF.os_inherit.enabled:
-            return [x['id'] for x in (
-                self.resource_api.list_project_parents(project_id))]
-        else:
-            return []
-
     @MEMOIZE_COMPUTED_ASSIGNMENTS
     def get_roles_for_user_and_project(self, user_id, tenant_id):
         """Get the roles associated with a user within given project.
@@ -165,11 +126,6 @@ class Manager(manager.Manager):
 
     def get_roles_for_groups(self, group_ids, project_id=None, domain_id=None):
         """Get a list of roles for this group on domain and/or project."""
-        # if no group ids were passed, there are no roles. Without this check,
-        # all assignments for the project or domain will be fetched,
-        # which is not what we want.
-        if not group_ids:
-            return []
         if project_id is not None:
             self.resource_api.get_project(project_id)
             assignment_list = self.list_role_assignments(
@@ -384,11 +340,11 @@ class Manager(manager.Manager):
             user_id, group_id, domain_id, project_id, inherited_to_projects)
         return self.role_api.list_roles_from_ids(grant_ids)
 
-    @notifications.role_assignment('deleted')
     def _emit_revoke_user_grant(self, role_id, user_id, domain_id, project_id,
                                 inherited_to_projects, context):
         self._emit_invalidate_grant_token_persistence(user_id, project_id)
 
+    @notifications.role_assignment('deleted')
     def delete_grant(self, role_id, user_id=None, group_id=None,
                      domain_id=None, project_id=None,
                      inherited_to_projects=False, context=None):
@@ -966,16 +922,7 @@ class Manager(manager.Manager):
         This stripping can be disabled by specifying strip_domain_roles=False,
         which is useful for internal calls like trusts which need to examine
         the full set of roles.
-
-        If OS-INHERIT extension is disabled or the used driver does not support
-        inherited roles retrieval, inherited role assignments will be ignored.
-
         """
-        if not CONF.os_inherit.enabled:
-            if inherited:
-                return []
-            inherited = False
-
         subtree_ids = None
         if project_id and include_subtree:
             subtree_ids = (
@@ -1000,40 +947,40 @@ class Manager(manager.Manager):
         role_assign_list = []
 
         for role_asgmt in role_assignments:
-            new_assign = {}
-            for id_type, id_ in role_asgmt.items():
-                if id_type == 'domain_id':
-                    _domain = self.resource_api.get_domain(id_)
-                    new_assign['domain_id'] = _domain['id']
+            new_assign = copy.deepcopy(role_asgmt)
+            for key, value in role_asgmt.items():
+                if key == 'domain_id':
+                    _domain = self.resource_api.get_domain(value)
                     new_assign['domain_name'] = _domain['name']
-                elif id_type == 'user_id':
-                    _user = self.identity_api.get_user(id_)
-                    new_assign['user_id'] = _user['id']
+                elif key == 'user_id':
+                    _user = self.identity_api.get_user(value)
                     new_assign['user_name'] = _user['name']
                     new_assign['user_domain_id'] = _user['domain_id']
                     new_assign['user_domain_name'] = (
                         self.resource_api.get_domain(_user['domain_id'])
                         ['name'])
-                elif id_type == 'group_id':
-                    _group = self.identity_api.get_group(id_)
-                    new_assign['group_id'] = _group['id']
+                elif key == 'group_id':
+                    _group = self.identity_api.get_group(value)
                     new_assign['group_name'] = _group['name']
                     new_assign['group_domain_id'] = _group['domain_id']
                     new_assign['group_domain_name'] = (
                         self.resource_api.get_domain(_group['domain_id'])
                         ['name'])
-                elif id_type == 'project_id':
-                    _project = self.resource_api.get_project(id_)
-                    new_assign['project_id'] = _project['id']
+                elif key == 'project_id':
+                    _project = self.resource_api.get_project(value)
                     new_assign['project_name'] = _project['name']
                     new_assign['project_domain_id'] = _project['domain_id']
                     new_assign['project_domain_name'] = (
                         self.resource_api.get_domain(_project['domain_id'])
                         ['name'])
-                elif id_type == 'role_id':
-                    _role = self.role_api.get_role(id_)
-                    new_assign['role_id'] = _role['id']
+                elif key == 'role_id':
+                    _role = self.role_api.get_role(value)
                     new_assign['role_name'] = _role['name']
+                    if _role['domain_id'] is not None:
+                        new_assign['role_domain_id'] = _role['domain_id']
+                        new_assign['role_domain_name'] = (
+                            self.resource_api.get_domain(_role['domain_id'])
+                            ['name'])
             role_assign_list.append(new_assign)
         return role_assign_list
 
@@ -1104,46 +1051,6 @@ class Manager(manager.Manager):
             )
 
 
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.assignment.AssignmentDriverBase',
-    in_favor_of='keystone.assignment.backends.base.AssignmentDriverBase',
-    remove_in=+1)
-class AssignmentDriverBase(base.AssignmentDriverBase):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.assignment.AssignmentDriverV8',
-    in_favor_of='keystone.assignment.backends.base.AssignmentDriverV8',
-    remove_in=+1)
-class AssignmentDriverV8(base.AssignmentDriverV8):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.assignment.AssignmentDriverV9',
-    in_favor_of='keystone.assignment.backends.base.AssignmentDriverV9',
-    remove_in=+1)
-class AssignmentDriverV9(base.AssignmentDriverV9):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.assignment.V9AssignmentWrapperForV8Driver',
-    in_favor_of=(
-        'keystone.assignment.backends.base.V9AssignmentWrapperForV8Driver'),
-    remove_in=+1)
-class V9AssignmentWrapperForV8Driver(base.V9AssignmentWrapperForV8Driver):
-    pass
-
-
-Driver = manager.create_legacy_driver(base.AssignmentDriverV8)
-
-
 @dependency.provider('role_api')
 @dependency.requires('assignment_api')
 class RoleManager(manager.Manager):
@@ -1164,13 +1071,6 @@ class RoleManager(manager.Manager):
 
         super(RoleManager, self).__init__(role_driver)
 
-        # Make sure it is a driver version we support, and if it is a legacy
-        # driver, then wrap it.
-        if isinstance(self.driver, role_base.RoleDriverV8):
-            self.driver = role_base.V9RoleWrapperForV8Driver(self.driver)
-        elif not isinstance(self.driver, role_base.RoleDriverV9):
-            raise exception.UnsupportedDriverVersion(driver=role_driver)
-
     def _append_null_domain_id(f):
         """Append a domain_id field to a role dict if it is not already there.
 
@@ -1249,43 +1149,3 @@ class RoleManager(manager.Manager):
     def delete_implied_role(self, prior_role_id, implied_role_id):
         self.driver.delete_implied_role(prior_role_id, implied_role_id)
         COMPUTED_ASSIGNMENTS_REGION.invalidate()
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.assignment.RoleDriverBase',
-    in_favor_of='keystone.assignment.role_backends.base.RoleDriverBase',
-    remove_in=+1)
-class RoleDriverBase(role_base.RoleDriverBase):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.assignment.RoleDriverV8',
-    in_favor_of='keystone.assignment.role_backends.base.RoleDriverV8',
-    remove_in=+1)
-class RoleDriverV8(role_base.RoleDriverV8):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.assignment.RoleDriverV9',
-    in_favor_of='keystone.assignment.role_backends.base.RoleDriverV9',
-    remove_in=+1)
-class RoleDriverV9(role_base.RoleDriverV9):
-    pass
-
-
-@versionutils.deprecated(
-    versionutils.deprecated.NEWTON,
-    what='keystone.assignment.V9RoleWrapperForV8Driver',
-    in_favor_of=(
-        'keystone.assignment.role_backends.base.V9RoleWrapperForV8Driver'),
-    remove_in=+1)
-class V9RoleWrapperForV8Driver(role_base.V9RoleWrapperForV8Driver):
-    pass
-
-
-RoleDriver = manager.create_legacy_driver(role_base.RoleDriverV8)
