@@ -14,15 +14,15 @@
 
 import sys
 
-from oslo_config import cfg
 from oslo_log import log
 import six
 
 from keystone.common import dependency
+import keystone.conf
 from keystone import exception
 
 
-CONF = cfg.CONF
+CONF = keystone.conf.CONF
 LOG = log.getLogger(__name__)
 
 
@@ -136,7 +136,6 @@ class BaseUserInfo(object):
     def _lookup_domain(self, domain_info):
         domain_id = domain_info.get('id')
         domain_name = domain_info.get('name')
-        domain_ref = None
         if not domain_id and not domain_name:
             raise exception.ValidationError(attribute='id or name',
                                             target='domain')
@@ -147,7 +146,7 @@ class BaseUserInfo(object):
             else:
                 domain_ref = self.resource_api.get_domain(domain_id)
         except exception.DomainNotFound as e:
-            LOG.exception(six.text_type(e))
+            LOG.warning(six.text_type(e))
             raise exception.Unauthorized(e)
         self._assert_domain_is_enabled(domain_ref)
         return domain_ref
@@ -159,7 +158,6 @@ class BaseUserInfo(object):
         user_info = auth_payload['user']
         user_id = user_info.get('id')
         user_name = user_info.get('name')
-        user_ref = None
         if not user_id and not user_name:
             raise exception.ValidationError(attribute='id or name',
                                             target='user')
@@ -177,7 +175,7 @@ class BaseUserInfo(object):
                     user_ref['domain_id'])
                 self._assert_domain_is_enabled(domain_ref)
         except exception.UserNotFound as e:
-            LOG.exception(six.text_type(e))
+            LOG.warning(six.text_type(e))
             raise exception.Unauthorized(e)
         self._assert_user_is_enabled(user_ref)
         self.user_ref = user_ref
