@@ -15,12 +15,12 @@
 
 import uuid
 
-import six
+from six.moves import http_client
 
-from keystone.catalog import core
 from keystone.catalog import schema
 from keystone.common import controller
 from keystone.common import dependency
+from keystone.common import utils
 from keystone.common import validation
 from keystone.common import wsgi
 from keystone import exception
@@ -36,32 +36,33 @@ INTERFACES = ['public', 'internal', 'admin']
 class Service(controller.V2Controller):
 
     @controller.v2_deprecated
-    def get_services(self, context):
-        self.assert_admin(context)
+    def get_services(self, request):
+        self.assert_admin(request)
         service_list = self.catalog_api.list_services()
         return {'OS-KSADM:services': service_list}
 
     @controller.v2_deprecated
-    def get_service(self, context, service_id):
-        self.assert_admin(context)
+    def get_service(self, request, service_id):
+        self.assert_admin(request)
         service_ref = self.catalog_api.get_service(service_id)
         return {'OS-KSADM:service': service_ref}
 
     @controller.v2_deprecated
-    def delete_service(self, context, service_id):
-        self.assert_admin(context)
-        initiator = notifications._get_request_audit_info(context)
-        self.catalog_api.delete_service(service_id, initiator)
+    def delete_service(self, request, service_id):
+        self.assert_admin(request)
+        self.catalog_api.delete_service(
+            service_id, initiator=request.audit_initiator
+        )
 
     @controller.v2_deprecated
-    def create_service(self, context, OS_KSADM_service):
-        self.assert_admin(context)
+    def create_service(self, request, OS_KSADM_service):
+        validation.lazy_validate(schema.service_create_v2, OS_KSADM_service)
+        self.assert_admin(request)
         service_id = uuid.uuid4().hex
         service_ref = OS_KSADM_service.copy()
         service_ref['id'] = service_id
-        initiator = notifications._get_request_audit_info(context)
         new_service_ref = self.catalog_api.create_service(
-            service_id, service_ref, initiator)
+            service_id, service_ref, initiator=request.audit_initiator)
         return {'OS-KSADM:service': new_service_ref}
 
 
@@ -69,9 +70,9 @@ class Service(controller.V2Controller):
 class Endpoint(controller.V2Controller):
 
     @controller.v2_deprecated
-    def get_endpoints(self, context):
+    def get_endpoints(self, request):
         """Merge matching v3 endpoint refs into legacy refs."""
-        self.assert_admin(context)
+        self.assert_admin(request)
         legacy_endpoints = {}
         v3_endpoints = {}
         for endpoint in self.catalog_api.list_endpoints():
@@ -129,9 +130,9 @@ class Endpoint(controller.V2Controller):
         return {'endpoints': list(legacy_endpoints.values())}
 
     @controller.v2_deprecated
-    def create_endpoint(self, context, endpoint):
+    def create_endpoint(self, request, endpoint):
         """Create three v3 endpoint refs based on a legacy ref."""
-        self.assert_admin(context)
+        self.assert_admin(request)
 
         # according to the v2 spec publicurl is mandatory
         self._require_attribute(endpoint, 'publicurl')
@@ -144,16 +145,16 @@ class Endpoint(controller.V2Controller):
         for interface in INTERFACES:
             interface_url = endpoint.get(interface + 'url')
             if interface_url:
-                core.check_endpoint_url(interface_url)
-
-        initiator = notifications._get_request_audit_info(context)
+                utils.check_endpoint_url(interface_url)
 
         if endpoint.get('region') is not None:
             try:
                 self.catalog_api.get_region(endpoint['region'])
             except exception.RegionNotFound:
                 region = dict(id=endpoint['region'])
-                self.catalog_api.create_region(region, initiator)
+                self.catalog_api.create_region(
+                    region, initiator=request.audit_initiator
+                )
 
         legacy_endpoint_ref = endpoint.copy()
 
@@ -176,23 +177,26 @@ class Endpoint(controller.V2Controller):
             endpoint_ref['legacy_endpoint_id'] = legacy_endpoint_id
             endpoint_ref['interface'] = interface
             endpoint_ref['url'] = url
-            endpoint_ref['region_id'] = endpoint_ref.pop('region')
-            self.catalog_api.create_endpoint(endpoint_ref['id'], endpoint_ref,
-                                             initiator)
+            endpoint_ref['region_id'] = endpoint_ref.pop('region', None)
+            self.catalog_api.create_endpoint(endpoint_ref['id'],
+                                             endpoint_ref,
+                                             initiator=request.audit_initiator)
 
         legacy_endpoint_ref['id'] = legacy_endpoint_id
         return {'endpoint': legacy_endpoint_ref}
 
     @controller.v2_deprecated
-    def delete_endpoint(self, context, endpoint_id):
+    def delete_endpoint(self, request, endpoint_id):
         """Delete up to three v3 endpoint refs based on a legacy ref ID."""
-        self.assert_admin(context)
-        initiator = notifications._get_request_audit_info(context)
+        self.assert_admin(request)
 
         deleted_at_least_one = False
         for endpoint in self.catalog_api.list_endpoints():
             if endpoint['legacy_endpoint_id'] == endpoint_id:
-                self.catalog_api.delete_endpoint(endpoint['id'], initiator)
+                self.catalog_api.delete_endpoint(
+                    endpoint['id'],
+                    initiator=request.audit_initiator
+                )
                 deleted_at_least_one = True
 
         if not deleted_at_least_one:
@@ -204,7 +208,7 @@ class RegionV3(controller.V3Controller):
     collection_name = 'regions'
     member_name = 'region'
 
-    def create_region_with_id(self, context, region_id, region):
+    def create_region_with_id(self, request, region_id, region):
         """Create a region with a user-specified ID.
 
         This method is unprotected because it depends on ``self.create_region``
@@ -217,45 +221,51 @@ class RegionV3(controller.V3Controller):
                       'url_id': region_id,
                       'ref_id': region['id']})
         region['id'] = region_id
-        return self.create_region(context, region)
+        return self.create_region(request, region)
 
     @controller.protected()
-    @validation.validated(schema.region_create, 'region')
-    def create_region(self, context, region):
+    def create_region(self, request, region):
+        validation.lazy_validate(schema.region_create, region)
         ref = self._normalize_dict(region)
 
         if not ref.get('id'):
             ref = self._assign_unique_id(ref)
 
-        initiator = notifications._get_request_audit_info(context)
-        ref = self.catalog_api.create_region(ref, initiator)
+        ref = self.catalog_api.create_region(
+            ref, initiator=request.audit_initiator
+        )
         return wsgi.render_response(
-            RegionV3.wrap_member(context, ref),
-            status=(201, 'Created'))
+            RegionV3.wrap_member(request.context_dict, ref),
+            status=(http_client.CREATED,
+                    http_client.responses[http_client.CREATED]))
 
     @controller.filterprotected('parent_region_id')
-    def list_regions(self, context, filters):
-        hints = RegionV3.build_driver_hints(context, filters)
+    def list_regions(self, request, filters):
+        hints = RegionV3.build_driver_hints(request, filters)
         refs = self.catalog_api.list_regions(hints)
-        return RegionV3.wrap_collection(context, refs, hints=hints)
+        return RegionV3.wrap_collection(request.context_dict,
+                                        refs,
+                                        hints=hints)
 
     @controller.protected()
-    def get_region(self, context, region_id):
+    def get_region(self, request, region_id):
         ref = self.catalog_api.get_region(region_id)
-        return RegionV3.wrap_member(context, ref)
+        return RegionV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
-    @validation.validated(schema.region_update, 'region')
-    def update_region(self, context, region_id, region):
+    def update_region(self, request, region_id, region):
+        validation.lazy_validate(schema.region_update, region)
         self._require_matching_id(region_id, region)
-        initiator = notifications._get_request_audit_info(context)
-        ref = self.catalog_api.update_region(region_id, region, initiator)
-        return RegionV3.wrap_member(context, ref)
+        ref = self.catalog_api.update_region(region_id,
+                                             region,
+                                             initiator=request.audit_initiator)
+        return RegionV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
-    def delete_region(self, context, region_id):
-        initiator = notifications._get_request_audit_info(context)
-        return self.catalog_api.delete_region(region_id, initiator)
+    def delete_region(self, request, region_id):
+        return self.catalog_api.delete_region(
+            region_id, initiator=request.audit_initiator
+        )
 
 
 @dependency.requires('catalog_api')
@@ -268,36 +278,41 @@ class ServiceV3(controller.V3Controller):
         self.get_member_from_driver = self.catalog_api.get_service
 
     @controller.protected()
-    @validation.validated(schema.service_create, 'service')
-    def create_service(self, context, service):
+    def create_service(self, request, service):
+        validation.lazy_validate(schema.service_create, service)
         ref = self._assign_unique_id(self._normalize_dict(service))
-        initiator = notifications._get_request_audit_info(context)
-        ref = self.catalog_api.create_service(ref['id'], ref, initiator)
-        return ServiceV3.wrap_member(context, ref)
+        ref = self.catalog_api.create_service(
+            ref['id'], ref, initiator=request.audit_initiator
+        )
+        return ServiceV3.wrap_member(request.context_dict, ref)
 
     @controller.filterprotected('type', 'name')
-    def list_services(self, context, filters):
-        hints = ServiceV3.build_driver_hints(context, filters)
+    def list_services(self, request, filters):
+        hints = ServiceV3.build_driver_hints(request, filters)
         refs = self.catalog_api.list_services(hints=hints)
-        return ServiceV3.wrap_collection(context, refs, hints=hints)
+        return ServiceV3.wrap_collection(request.context_dict,
+                                         refs,
+                                         hints=hints)
 
     @controller.protected()
-    def get_service(self, context, service_id):
+    def get_service(self, request, service_id):
         ref = self.catalog_api.get_service(service_id)
-        return ServiceV3.wrap_member(context, ref)
+        return ServiceV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
-    @validation.validated(schema.service_update, 'service')
-    def update_service(self, context, service_id, service):
+    def update_service(self, request, service_id, service):
+        validation.lazy_validate(schema.service_update, service)
         self._require_matching_id(service_id, service)
-        initiator = notifications._get_request_audit_info(context)
-        ref = self.catalog_api.update_service(service_id, service, initiator)
-        return ServiceV3.wrap_member(context, ref)
+        ref = self.catalog_api.update_service(
+            service_id, service, initiator=request.audit_initiator
+        )
+        return ServiceV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
-    def delete_service(self, context, service_id):
-        initiator = notifications._get_request_audit_info(context)
-        return self.catalog_api.delete_service(service_id, initiator)
+    def delete_service(self, request, service_id):
+        return self.catalog_api.delete_service(
+            service_id, initiator=request.audit_initiator
+        )
 
 
 @dependency.requires('catalog_api')
@@ -321,7 +336,7 @@ class EndpointV3(controller.V3Controller):
         ref = cls.filter_endpoint(ref)
         return super(EndpointV3, cls).wrap_member(context, ref)
 
-    def _validate_endpoint_region(self, endpoint, context=None):
+    def _validate_endpoint_region(self, endpoint, request):
         """Ensure the region for the endpoint exists.
 
         If 'region_id' is used to specify the region, then we will let the
@@ -340,48 +355,54 @@ class EndpointV3(controller.V3Controller):
                 self.catalog_api.get_region(endpoint['region_id'])
             except exception.RegionNotFound:
                 region = dict(id=endpoint['region_id'])
-                initiator = notifications._get_request_audit_info(context)
-                self.catalog_api.create_region(region, initiator)
+                self.catalog_api.create_region(
+                    region, initiator=request.audit_initiator
+                )
 
         return endpoint
 
     @controller.protected()
-    @validation.validated(schema.endpoint_create, 'endpoint')
-    def create_endpoint(self, context, endpoint):
-        core.check_endpoint_url(endpoint['url'])
+    def create_endpoint(self, request, endpoint):
+        validation.lazy_validate(schema.endpoint_create, endpoint)
+        utils.check_endpoint_url(endpoint['url'])
         ref = self._assign_unique_id(self._normalize_dict(endpoint))
-        ref = self._validate_endpoint_region(ref, context)
-        initiator = notifications._get_request_audit_info(context)
-        ref = self.catalog_api.create_endpoint(ref['id'], ref, initiator)
-        return EndpointV3.wrap_member(context, ref)
+        ref = self._validate_endpoint_region(ref, request)
+        ref = self.catalog_api.create_endpoint(
+            ref['id'], ref, initiator=request.audit_initiator
+        )
+        return EndpointV3.wrap_member(request.context_dict, ref)
 
     @controller.filterprotected('interface', 'service_id', 'region_id')
-    def list_endpoints(self, context, filters):
-        hints = EndpointV3.build_driver_hints(context, filters)
+    def list_endpoints(self, request, filters):
+        hints = EndpointV3.build_driver_hints(request, filters)
         refs = self.catalog_api.list_endpoints(hints=hints)
-        return EndpointV3.wrap_collection(context, refs, hints=hints)
+        return EndpointV3.wrap_collection(request.context_dict,
+                                          refs,
+                                          hints=hints)
 
     @controller.protected()
-    def get_endpoint(self, context, endpoint_id):
+    def get_endpoint(self, request, endpoint_id):
         ref = self.catalog_api.get_endpoint(endpoint_id)
-        return EndpointV3.wrap_member(context, ref)
+        return EndpointV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
-    @validation.validated(schema.endpoint_update, 'endpoint')
-    def update_endpoint(self, context, endpoint_id, endpoint):
+    def update_endpoint(self, request, endpoint_id, endpoint):
+        validation.lazy_validate(schema.endpoint_update, endpoint)
         self._require_matching_id(endpoint_id, endpoint)
 
-        endpoint = self._validate_endpoint_region(endpoint.copy(), context)
+        endpoint = self._validate_endpoint_region(endpoint.copy(),
+                                                  request)
 
-        initiator = notifications._get_request_audit_info(context)
-        ref = self.catalog_api.update_endpoint(endpoint_id, endpoint,
-                                               initiator)
-        return EndpointV3.wrap_member(context, ref)
+        ref = self.catalog_api.update_endpoint(
+            endpoint_id, endpoint, initiator=request.audit_initiator
+        )
+        return EndpointV3.wrap_member(request.context_dict, ref)
 
     @controller.protected()
-    def delete_endpoint(self, context, endpoint_id):
-        initiator = notifications._get_request_audit_info(context)
-        return self.catalog_api.delete_endpoint(endpoint_id, initiator)
+    def delete_endpoint(self, request, endpoint_id):
+        return self.catalog_api.delete_endpoint(
+            endpoint_id, initiator=request.audit_initiator
+        )
 
 
 @dependency.requires('catalog_api', 'resource_api')
@@ -400,23 +421,15 @@ class EndpointFilterV3Controller(controller.V3Controller):
                                        payload):
         project_or_endpoint_id = payload['resource_info']
         if resource_type == 'project':
-            try:
-                self.catalog_api.delete_association_by_project(
-                    project_or_endpoint_id)
-            except exception.NotImplemented:
-                # Some catalog drivers don't support this
-                pass
+            self.catalog_api.delete_association_by_project(
+                project_or_endpoint_id)
         else:
-            try:
-                self.catalog_api.delete_association_by_endpoint(
-                    project_or_endpoint_id)
-            except exception.NotImplemented:
-                # Some catalog drivers don't support this
-                pass
+            self.catalog_api.delete_association_by_endpoint(
+                project_or_endpoint_id)
 
     @controller.protected()
-    def add_endpoint_to_project(self, context, project_id, endpoint_id):
-        """Establishes an association between an endpoint and a project."""
+    def add_endpoint_to_project(self, request, project_id, endpoint_id):
+        """Establish an association between an endpoint and a project."""
         # NOTE(gyee): we just need to make sure endpoint and project exist
         # first. We don't really care whether if project is disabled.
         # The relationship can still be established even with a disabled
@@ -427,39 +440,40 @@ class EndpointFilterV3Controller(controller.V3Controller):
                                                  project_id)
 
     @controller.protected()
-    def check_endpoint_in_project(self, context, project_id, endpoint_id):
-        """Verifies endpoint is currently associated with given project."""
+    def check_endpoint_in_project(self, request, project_id, endpoint_id):
+        """Verify endpoint is currently associated with given project."""
         self.catalog_api.get_endpoint(endpoint_id)
         self.resource_api.get_project(project_id)
         self.catalog_api.check_endpoint_in_project(endpoint_id,
                                                    project_id)
 
     @controller.protected()
-    def list_endpoints_for_project(self, context, project_id):
+    def list_endpoints_for_project(self, request, project_id):
         """List all endpoints currently associated with a given project."""
         self.resource_api.get_project(project_id)
         filtered_endpoints = self.catalog_api.list_endpoints_for_project(
             project_id)
 
         return EndpointV3.wrap_collection(
-            context, [v for v in six.itervalues(filtered_endpoints)])
+            request.context_dict,
+            [v for v in filtered_endpoints.values()])
 
     @controller.protected()
-    def remove_endpoint_from_project(self, context, project_id, endpoint_id):
+    def remove_endpoint_from_project(self, request, project_id, endpoint_id):
         """Remove the endpoint from the association with given project."""
         self.catalog_api.remove_endpoint_from_project(endpoint_id,
                                                       project_id)
 
     @controller.protected()
-    def list_projects_for_endpoint(self, context, endpoint_id):
+    def list_projects_for_endpoint(self, request, endpoint_id):
         """Return a list of projects associated with the endpoint."""
         self.catalog_api.get_endpoint(endpoint_id)
         refs = self.catalog_api.list_projects_for_endpoint(endpoint_id)
 
         projects = [self.resource_api.get_project(
             ref['project_id']) for ref in refs]
-        return resource.controllers.ProjectV3.wrap_collection(context,
-                                                              projects)
+        return resource.controllers.ProjectV3.wrap_collection(
+            request.context_dict, projects)
 
 
 @dependency.requires('catalog_api', 'resource_api')
@@ -480,18 +494,18 @@ class EndpointGroupV3Controller(controller.V3Controller):
                                                               path=path)
 
     @controller.protected()
-    @validation.validated(schema.endpoint_group_create, 'endpoint_group')
-    def create_endpoint_group(self, context, endpoint_group):
-        """Creates an Endpoint Group with the associated filters."""
+    def create_endpoint_group(self, request, endpoint_group):
+        """Create an Endpoint Group with the associated filters."""
+        validation.lazy_validate(schema.endpoint_group_create, endpoint_group)
         ref = self._assign_unique_id(self._normalize_dict(endpoint_group))
         self._require_attribute(ref, 'filters')
         self._require_valid_filter(ref)
         ref = self.catalog_api.create_endpoint_group(ref['id'], ref)
-        return EndpointGroupV3Controller.wrap_member(context, ref)
+        return EndpointGroupV3Controller.wrap_member(request.context_dict, ref)
 
     def _require_valid_filter(self, endpoint_group):
         filters = endpoint_group.get('filters')
-        for key in six.iterkeys(filters):
+        for key in filters.keys():
             if key not in self.VALID_FILTER_KEYS:
                 raise exception.ValidationError(
                     attribute=self._valid_filter_keys(),
@@ -501,46 +515,46 @@ class EndpointGroupV3Controller(controller.V3Controller):
         return ' or '.join(self.VALID_FILTER_KEYS)
 
     @controller.protected()
-    def get_endpoint_group(self, context, endpoint_group_id):
+    def get_endpoint_group(self, request, endpoint_group_id):
         """Retrieve the endpoint group associated with the id if exists."""
         ref = self.catalog_api.get_endpoint_group(endpoint_group_id)
         return EndpointGroupV3Controller.wrap_member(
-            context, ref)
+            request.context_dict, ref)
 
     @controller.protected()
-    @validation.validated(schema.endpoint_group_update, 'endpoint_group')
-    def update_endpoint_group(self, context, endpoint_group_id,
+    def update_endpoint_group(self, request, endpoint_group_id,
                               endpoint_group):
         """Update fixed values and/or extend the filters."""
+        validation.lazy_validate(schema.endpoint_group_update, endpoint_group)
         if 'filters' in endpoint_group:
             self._require_valid_filter(endpoint_group)
         ref = self.catalog_api.update_endpoint_group(endpoint_group_id,
                                                      endpoint_group)
         return EndpointGroupV3Controller.wrap_member(
-            context, ref)
+            request.context_dict, ref)
 
     @controller.protected()
-    def delete_endpoint_group(self, context, endpoint_group_id):
+    def delete_endpoint_group(self, request, endpoint_group_id):
         """Delete endpoint_group."""
         self.catalog_api.delete_endpoint_group(endpoint_group_id)
 
     @controller.protected()
-    def list_endpoint_groups(self, context):
+    def list_endpoint_groups(self, request):
         """List all endpoint groups."""
         refs = self.catalog_api.list_endpoint_groups()
         return EndpointGroupV3Controller.wrap_collection(
-            context, refs)
+            request.context_dict, refs)
 
     @controller.protected()
-    def list_endpoint_groups_for_project(self, context, project_id):
+    def list_endpoint_groups_for_project(self, request, project_id):
         """List all endpoint groups associated with a given project."""
         return EndpointGroupV3Controller.wrap_collection(
-            context,
+            request.context_dict,
             self.catalog_api.get_endpoint_groups_for_project(project_id))
 
     @controller.protected()
     def list_projects_associated_with_endpoint_group(self,
-                                                     context,
+                                                     request,
                                                      endpoint_group_id):
         """List all projects associated with endpoint group."""
         endpoint_group_refs = (self.catalog_api.
@@ -552,18 +566,19 @@ class EndpointGroupV3Controller(controller.V3Controller):
                 endpoint_group_ref['project_id'])
             if project:
                 projects.append(project)
-        return resource.controllers.ProjectV3.wrap_collection(context,
-                                                              projects)
+        return resource.controllers.ProjectV3.wrap_collection(
+            request.context_dict, projects)
 
     @controller.protected()
     def list_endpoints_associated_with_endpoint_group(self,
-                                                      context,
+                                                      request,
                                                       endpoint_group_id):
         """List all the endpoints filtered by a specific endpoint group."""
         filtered_endpoints = (self.catalog_api.
                               get_endpoints_filtered_by_endpoint_group(
                                   endpoint_group_id))
-        return EndpointV3.wrap_collection(context, filtered_endpoints)
+        return EndpointV3.wrap_collection(request.context_dict,
+                                          filtered_endpoints)
 
 
 @dependency.requires('catalog_api', 'resource_api')
@@ -580,16 +595,11 @@ class ProjectEndpointGroupV3Controller(controller.V3Controller):
     def _on_project_delete(self, service, resource_type,
                            operation, payload):
         project_id = payload['resource_info']
-        try:
-            (self.catalog_api.
-             delete_endpoint_group_association_by_project(
-                 project_id))
-        except exception.NotImplemented:
-            # Some catalog drivers don't support this
-            pass
+        self.catalog_api.delete_endpoint_group_association_by_project(
+            project_id)
 
     @controller.protected()
-    def get_endpoint_group_in_project(self, context, endpoint_group_id,
+    def get_endpoint_group_in_project(self, request, endpoint_group_id,
                                       project_id):
         """Retrieve the endpoint group associated with the id if exists."""
         self.resource_api.get_project(project_id)
@@ -597,19 +607,19 @@ class ProjectEndpointGroupV3Controller(controller.V3Controller):
         ref = self.catalog_api.get_endpoint_group_in_project(
             endpoint_group_id, project_id)
         return ProjectEndpointGroupV3Controller.wrap_member(
-            context, ref)
+            request.context_dict, ref)
 
     @controller.protected()
-    def add_endpoint_group_to_project(self, context, endpoint_group_id,
+    def add_endpoint_group_to_project(self, request, endpoint_group_id,
                                       project_id):
-        """Creates an association between an endpoint group and project."""
+        """Create an association between an endpoint group and project."""
         self.resource_api.get_project(project_id)
         self.catalog_api.get_endpoint_group(endpoint_group_id)
         self.catalog_api.add_endpoint_group_to_project(
             endpoint_group_id, project_id)
 
     @controller.protected()
-    def remove_endpoint_group_from_project(self, context, endpoint_group_id,
+    def remove_endpoint_group_from_project(self, request, endpoint_group_id,
                                            project_id):
         """Remove the endpoint group from associated project."""
         self.resource_api.get_project(project_id)
