@@ -27,9 +27,10 @@ import pbr.version
 
 from keystone.cmd import doctor
 from keystone.common import driver_hints
+from keystone.common import fernet_utils
 from keystone.common import openssl
 from keystone.common import sql
-from keystone.common.sql import migration_helpers
+from keystone.common.sql import upgrades
 from keystone.common import utils
 import keystone.conf
 from keystone.credential.providers import fernet as credential_fernet
@@ -450,14 +451,19 @@ class DbSync(BaseApp):
     def main():
         assert_not_extension(CONF.command.extension)
 
-        if CONF.command.expand:
-            migration_helpers.expand_schema()
+        # It is possible to run expand and migrate at the same time,
+        # expand needs to run first however.
+        if CONF.command.expand and CONF.command.migrate:
+            upgrades.expand_schema()
+            upgrades.migrate_data()
+        elif CONF.command.expand:
+            upgrades.expand_schema()
         elif CONF.command.migrate:
-            migration_helpers.migrate_data()
+            upgrades.migrate_data()
         elif CONF.command.contract:
-            migration_helpers.contract_schema()
+            upgrades.contract_schema()
         else:
-            migration_helpers.offline_sync_database_to_version(
+            upgrades.offline_sync_database_to_version(
                 CONF.command.version)
 
 
@@ -480,7 +486,7 @@ class DbVersion(BaseApp):
     @staticmethod
     def main():
         assert_not_extension(CONF.command.extension)
-        migration_helpers.print_db_version()
+        print(upgrades.get_db_version())
 
 
 class BasePermissionsSetup(BaseApp):
@@ -531,7 +537,7 @@ class BaseCertificateSetup(BasePermissionsSetup):
 
 
 class PKISetup(BaseCertificateSetup):
-    """Set up Key pairs and certificates for token signing and verification.
+    """Setup keys and certificates for signing and verifying revocation lists.
 
     This is NOT intended for production use, see Keystone Configuration
     documentation for details. As of the Mitaka release, this command has
@@ -568,16 +574,16 @@ class FernetSetup(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
         )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        fernet_utils.create_key_directory(keystone_user_id, keystone_group_id)
-        if fernet_utils.validate_key_repository(requires_write=True):
-            fernet_utils.initialize_key_repository(
+        futils.create_key_directory(keystone_user_id, keystone_group_id)
+        if futils.validate_key_repository(requires_write=True):
+            futils.initialize_key_repository(
                 keystone_user_id, keystone_group_id)
 
 
@@ -603,15 +609,15 @@ class FernetRotate(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.fernet_tokens.key_repository,
-            CONF.fernet_tokens.max_active_keys
+            CONF.fernet_tokens.max_active_keys,
+            'fernet_tokens'
         )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        if fernet_utils.validate_key_repository(requires_write=True):
-            fernet_utils.rotate_keys(keystone_user_id, keystone_group_id)
+        if futils.validate_key_repository(requires_write=True):
+            futils.rotate_keys(keystone_user_id, keystone_group_id)
 
 
 class CredentialSetup(BasePermissionsSetup):
@@ -627,16 +633,16 @@ class CredentialSetup(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.credential.key_repository,
-            credential_fernet.MAX_ACTIVE_KEYS
+            credential_fernet.MAX_ACTIVE_KEYS,
+            'credential'
         )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        fernet_utils.create_key_directory(keystone_user_id, keystone_group_id)
-        if fernet_utils.validate_key_repository(requires_write=True):
-            fernet_utils.initialize_key_repository(
+        futils.create_key_directory(keystone_user_id, keystone_group_id)
+        if futils.validate_key_repository(requires_write=True):
+            futils.initialize_key_repository(
                 keystone_user_id,
                 keystone_group_id
             )
@@ -699,17 +705,17 @@ class CredentialRotate(BasePermissionsSetup):
 
     @classmethod
     def main(cls):
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.credential.key_repository,
-            credential_fernet.MAX_ACTIVE_KEYS
+            credential_fernet.MAX_ACTIVE_KEYS,
+            'credential'
         )
 
         keystone_user_id, keystone_group_id = cls.get_user_group()
-        if fernet_utils.validate_key_repository(requires_write=True):
+        if futils.validate_key_repository(requires_write=True):
             klass = cls()
             klass.validate_primary_key()
-            fernet_utils.rotate_keys(keystone_user_id, keystone_group_id)
+            futils.rotate_keys(keystone_user_id, keystone_group_id)
 
 
 class CredentialMigrate(BasePermissionsSetup):
@@ -759,12 +765,12 @@ class CredentialMigrate(BasePermissionsSetup):
     @classmethod
     def main(cls):
         # Check to make sure we have a repository that works...
-        from keystone.common import fernet_utils as utils
-        fernet_utils = utils.FernetUtils(
+        futils = fernet_utils.FernetUtils(
             CONF.credential.key_repository,
-            credential_fernet.MAX_ACTIVE_KEYS
+            credential_fernet.MAX_ACTIVE_KEYS,
+            'credential'
         )
-        fernet_utils.validate_key_repository(requires_write=True)
+        futils.validate_key_repository(requires_write=True)
         klass = cls()
         klass.migrate_credentials()
 
@@ -1011,9 +1017,21 @@ class DomainConfigUploadFiles(object):
                 return False
             return True
 
+        success_cnt = 0
+        failure_cnt = 0
         for filename, domain_name in self._domain_config_finder(conf_dir):
-            self._upload_config_to_database(filename, domain_name)
+            if self._upload_config_to_database(filename, domain_name):
+                success_cnt += 1
+                LOG.info(_LI('Successfully uploaded domain config %r'),
+                         filename)
+            else:
+                failure_cnt += 1
 
+        if success_cnt == 0:
+            LOG.warning(_LW('No domain configs uploaded from %r'), conf_dir)
+
+        if failure_cnt:
+            return False
         return True
 
     def run(self):
@@ -1083,65 +1101,89 @@ class MappingEngineTester(BaseApp):
 
     name = 'mapping_engine'
 
-    @staticmethod
-    def read_rules(path):
+    def __init__(self):
+        super(MappingEngineTester, self).__init__()
+        self.mapping_id = uuid.uuid4().hex
+        self.rules_pathname = None
+        self.rules = None
+        self.assertion_pathname = None
+        self.assertion = None
+
+    def read_rules(self, path):
+        self.rules_pathname = path
         try:
             with open(path) as file:
-                return jsonutils.load(file)
+                self.rules = jsonutils.load(file)
         except ValueError as e:
             raise SystemExit(_('Error while parsing rules '
                                '%(path)s: %(err)s') % {'path': path, 'err': e})
 
-    @staticmethod
-    def read_file(path):
+    def read_assertion(self, path):
+        self.assertion_pathname = path
         try:
             with open(path) as file:
-                return file.read().strip()
+                self.assertion = file.read().strip()
         except IOError as e:
             raise SystemExit(_("Error while opening file "
                                "%(path)s: %(err)s") % {'path': path, 'err': e})
 
-    @staticmethod
-    def normalize_assertion(assertion):
-        def split(line):
+    def normalize_assertion(self):
+        def split(line, line_num):
             try:
                 k, v = line.split(':', 1)
                 return k.strip(), v.strip()
-            except ValueError as e:
-                msg = _("Error while parsing line: '%(line)s': %(err)s")
-                raise SystemExit(msg % {'line': line, 'err': e})
-        assertion = assertion.split('\n')
+            except ValueError:
+                msg = _("assertion file %(pathname)s at line %(line_num)d "
+                        "expected 'key: value' but found '%(line)s' "
+                        "see help for file format")
+                raise SystemExit(msg % {'pathname': self.assertion_pathname,
+                                        'line_num': line_num,
+                                        'line': line})
+        assertion = self.assertion.split('\n')
         assertion_dict = {}
         prefix = CONF.command.prefix
-        for line in assertion:
-            k, v = split(line)
+        for line_num, line in enumerate(assertion, 1):
+            line = line.strip()
+            if line == '':
+                continue
+            k, v = split(line, line_num)
             if prefix:
                 if k.startswith(prefix):
                     assertion_dict[k] = v
             else:
                 assertion_dict[k] = v
-        return assertion_dict
+        self.assertion = assertion_dict
 
-    @staticmethod
-    def normalize_rules(rules):
-        if isinstance(rules, list):
-            return {'rules': rules}
-        else:
-            return rules
+    def normalize_rules(self):
+        if isinstance(self.rules, list):
+            self.rules = {'rules': self.rules}
 
     @classmethod
     def main(cls):
-        if not CONF.command.engine_debug:
+        if CONF.command.engine_debug:
+            mapping_engine.LOG.logger.setLevel('DEBUG')
+        else:
             mapping_engine.LOG.logger.setLevel('WARN')
 
-        rules = MappingEngineTester.read_rules(CONF.command.rules)
-        rules = MappingEngineTester.normalize_rules(rules)
-        mapping_engine.validate_mapping_structure(rules)
+        tester = cls()
 
-        assertion = MappingEngineTester.read_file(CONF.command.input)
-        assertion = MappingEngineTester.normalize_assertion(assertion)
-        rp = mapping_engine.RuleProcessor(rules['rules'])
-        print(jsonutils.dumps(rp.process(assertion), indent=2))
+        tester.read_rules(CONF.command.rules)
+        tester.normalize_rules()
+        mapping_engine.validate_mapping_structure(tester.rules)
+
+        tester.read_assertion(CONF.command.input)
+        tester.normalize_assertion()
+
+        if CONF.command.engine_debug:
+            print("Using Rules:\n%s" % (
+                jsonutils.dumps(tester.rules, indent=2)))
+            print("Using Assertion:\n%s" % (
+                jsonutils.dumps(tester.assertion, indent=2)))
+
+        rp = mapping_engine.RuleProcessor(tester.mapping_id,
+                                          tester.rules['rules'])
+        mapped = rp.process(tester.assertion)
+        print(jsonutils.dumps(mapped, indent=2))
 
     @classmethod
     def add_argument_parser(cls, subparsers):
@@ -1212,8 +1254,8 @@ class MappingPopulate(BaseApp):
         try:
             domain_id = cls.resource_api.get_domain_by_name(domain_name)['id']
         except exception.DomainNotFound:
-            print(_('Invalid domain name or ID: %(domain)s') % {
-                'domain': domain_id})
+            print(_('Invalid domain name: %(domain)s') % {
+                'domain': domain_name})
             return False
         # We don't actually need to tackle id_mapping_api in order to get
         # entries there, because list_users does this anyway. That's why it
