@@ -18,26 +18,26 @@ from keystone.i18n import _
 
 
 def build_v3_resource_relation(resource_name):
-    return ('http://docs.openstack.org/api/openstack-identity/3/rel/%s' %
+    return ('https://docs.openstack.org/api/openstack-identity/3/rel/%s' %
             resource_name)
 
 
 def build_v3_extension_resource_relation(extension_name, extension_version,
                                          resource_name):
     return (
-        'http://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/rel/%s' %
-        (extension_name, extension_version, resource_name))
+        'https://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/rel/'
+        '%s' % (extension_name, extension_version, resource_name))
 
 
 def build_v3_parameter_relation(parameter_name):
-    return ('http://docs.openstack.org/api/openstack-identity/3/param/%s' %
+    return ('https://docs.openstack.org/api/openstack-identity/3/param/%s' %
             parameter_name)
 
 
 def build_v3_extension_parameter_relation(extension_name, extension_version,
                                           parameter_name):
     return (
-        'http://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/param/'
+        'https://docs.openstack.org/api/openstack-identity/3/ext/%s/%s/param/'
         '%s' % (extension_name, extension_version, parameter_name))
 
 
